package org.robotlegs.utilities.modular.starling.mvcs 
{
	import flash.events.Event;
	import flash.system.ApplicationDomain;
	import org.robotlegs.base.ContextEvent;
	import org.robotlegs.core.IInjector;
	import org.robotlegs.mvcs.StarlingContext;
	import org.robotlegs.utilities.modular.base.ModuleCommandMap;
	import org.robotlegs.utilities.modular.base.ModuleEventDispatcher;
	import org.robotlegs.utilities.modular.core.IModuleCommandMap;
	import org.robotlegs.utilities.modular.core.IModuleContext;
	import org.robotlegs.utilities.modular.core.IModuleEventDispatcher;
	import starling.display.DisplayObjectContainer;
	
	/**
	 * ... Starling and ModuleContext integration ...
	 * @author Jerry
	 */
	public class StarlingModuleContext extends StarlingContext implements IModuleContext
	{
		/**
		 * The domain of application.
		 */
		protected var _applicationDomain:ApplicationDomain;
		
		/**
		 * The module event dispatcher.
		 */
		protected var _moduleDispatcher:IModuleEventDispatcher;
		
		/**
		 * Get the module event dispatcher
		 */
		protected function get moduleDispatcher():IModuleEventDispatcher
		{
			return _moduleDispatcher;
		}
		
		/**
		 * Set the module event dispatcher
		 */
		protected function set moduleDispatcher(value:IModuleEventDispatcher):void
		{
			_moduleDispatcher = value;
		}
		
		/**
		 * The module commands map
		 */
		protected var _moduleCommandMap:IModuleCommandMap;
		
		/**
		 * Get the module commands map
		 */
		protected function get moduleCommandMap():IModuleCommandMap
		{
			return _moduleCommandMap || (_moduleCommandMap = new ModuleCommandMap(moduleDispatcher, injector.createChild(_applicationDomain), reflector));
		}
		
		/**
		 * Set the module commands map
		 */
		protected function set moduleCommandMap(value:IModuleCommandMap):void
		{
			_moduleCommandMap = value;
		}
		
		/**
		 * Constructor Method
		 * @param	contextView
		 * @param	autoStartup
		 * @param	parentInjector
		 * @param	applicationDomain
		 */
		public function StarlingModuleContext(contextView:DisplayObjectContainer=null, autoStartup:Boolean=true, parentInjector:IInjector = null, applicationDomain:ApplicationDomain = null) 
		{
			_applicationDomain = applicationDomain || ApplicationDomain.currentDomain;
			
            if(parentInjector)
            {
                _injector = parentInjector.createChild(_applicationDomain);
            }
			
			super(contextView, autoStartup);
		}
		
		/**
		 * Map injectins object
		 */
		override protected function mapInjections():void
        {
            super.mapInjections();
            initializeModuleEventDispatcher();
            injector.mapValue(IModuleCommandMap, moduleCommandMap);
        }
		
		/**
		 * Initialize module event dispatcher
		 */
		protected function initializeModuleEventDispatcher():void
        {
            if(injector.hasMapping(IModuleEventDispatcher) )
            {
                moduleDispatcher = injector.getInstance(IModuleEventDispatcher);
            }
            else
            {
                moduleDispatcher = new ModuleEventDispatcher(this);
                injector.mapValue(IModuleEventDispatcher, moduleDispatcher); 
            }          
        }
		
		/**
		 * Dispatch event to all modules
		 * @param	event
		 * @return
		 */
		protected function dispatchToModules(event:Event):Boolean
        {
            if (moduleDispatcher.hasEventListener(event.type))
                return moduleDispatcher.dispatchEvent(event);
            return true;
		}
		
		/**
		 * Dispose objects
		 */
		public function dispose():void
        {
            dispatchEvent(new ContextEvent(ContextEvent.SHUTDOWN));
            _commandMap.unmapEvents();
            _moduleCommandMap.unmapEvents();
            _moduleCommandMap = null;
            _moduleDispatcher = null;
            _contextView = null;
            _injector = null;
            _reflector = null;
            _commandMap = null;
            _mediatorMap = null;
            _viewMap = null;
            _eventDispatcher = null;
            _applicationDomain = null;
        }
	}

}